# JRTC简介
JRTC的基本功能包含创建实例、加入房间、本地发布、订阅远端和离开房间等。当您成功创建实例，您可以进行本地预览视频功能，进行简单的预览和测试。

# 前提条件
在执行Demo步骤之前，您需要从控制台获取鉴权信息，具体操作请参见[生成Token](https://docs.jdcloud.com/cn/real-time-communication/sdk/generate-user-token)。
您需要下载示例代码，详情请参见[SDK下载](https://www.cnblogs.com/dotnetcrazy/p/9180295.html#_map7)。

# 操作步骤
1.集成SDK。
  > JRTC SDK存放于demo中src/lib文件夹内，可根据自身项目构建需求，调整JRTC SDK存放位置。
  ```js
  import JRTCClient from '../lib/jrtc.min.js'
  ``` 

2.初始化SDK。
  ```js
  const JWebrtc = JRTCClient.init({ appId })
  ```
  > **说明** appId 为应用ID，从控制台获取
  

  
3.加入房间。进入房间成功, 会返回一个ROOM对象: JRTCRoom。
   ```js
   JWebrtc.enterRoom({ 
         appId,           // 应用ID，从控制台获取
         token,           // 用户生成token，生成方式参见XXXX取
         userId,          // 用户ID
         nonce,           // 令牌随机码，用户生成
         timestamp,       // 令牌过期时间，用户生成
         roomId,          // 房间ID
        //  peerId,          // 会议平台的用户ID
         nickName,        // 昵称,可选，默认为 '游客'
         subscribeType,    // 可选参数,大房间模式下，音频订阅模式：1.固定订阅 2.普通订阅。 默认为 1
         useVp8            // 可选参数，是否开启vp8环境， 默认false
   }).then((res) => {
      // 成功加入会议
   }).catch((err) => {
      // 加入会议失败，打印错误日志，可以查看失败原因    
      console.log(err);
   });
   ```

4.发布或取消发布音、视频流。
  * 发布音频流。通过getAudioTrack方法获取音频轨道后，使用publishAudioStream方法可以发布音频流。
  ```js
  JWebrtc.getAudioTrack()  // 该方法返回 audioTrack
      .then((track) => {
        JRTCRoom.publishAudioStream(track).then(
          ({ track, streamId }) => {
            // 音频流发布成功
          }
        );
      })
      .catch((err) => {
        // 音频流发布失败， 打印错误日志，可以查看失败原因   
        console.log(err);
      });
  ```

  * 发布视频流。通过getVideoTrack方法获取视频轨道后，使用publishVideoStream方法可以发布视频流。
  ```js
  JWebrtc.getVideoTrack()    // 该方法返回 videoTrack
      .then((track) => {
        JRTCRoom.publishVideoStream(track).then(
          ({ track, streamId }) => {
            // 视频流发布成功
          }
        );
      })
      .catch((err) => {
        // 视频流发布失败， 打印错误日志，可以查看失败原因
        console.log(err);
      });
  ```

  * 取消发布音、视频流。
  ```js
  JRTCRoom.unPublishStream(streamId);   // streamId - 流ID
  ```

5.发布StreamPublished回调或StreamUnpublished回调。
  * 发布StreamPublished回调。当有用户发布新的音、视频流时，在SDK里会触发StreamPublished回调，通过订阅这个回调，能够得到房间里已经发布新流的用户。
  ```js
  JRTCRoom.on("StreamPublished", (streamInfos) => {
      // 当有新流发布时，可获取streamInfos
      console.log(streamInfos)
  });
  ```

streamInfos 信息如下表所示：
  
  | 参数     |  说明   |
| -------- | :------- |
| userId | 用户Id |
| nickName | 昵称 |
| kind | 流类型 |
| streamName | 流名称 |

  * 发布StreamUnpublished回调。当有用户取消音频或视频发布时，会触发这个回调。
  ```js
  JRTCRoom.on("StreamUnpublished", (peerInfo) => {
    // 返回取消发布的流信息
    console.log(peerInfo);
  });
  ```

peerInfo 信息如下表所示：
  
  | 参数     |  说明   |
| -------- | :------- |
| userId | 用户Id |
| nickName | 昵称 |
| kind | 流类型 |

  > **说明** StreamPublished、StreamUnpublished回调只有加入房间以后才会触发。

6.订阅或取消订阅流。
  * 订阅流。通过subscribeStreams方法可以订阅流，订阅成功如果产生新的消费者，需手动监听streamSubscribed。
  ```js
  JRTCRoom.subscribeStreams(streamIds);   // 订阅流，streamIds - 流ID Array
  ```
  > 手动监听 streamSubscribed，返回peerInfo。
  > ```js
  > JRTCRoom.on("StreamSubscribed", ({ peerInfo }) => {
  >    console.log("StreamSubscribed", peerInfo);
  > })
  > ```
  
  peerInfo 信息如下表所示：
  
  | 参数     |  说明   |
| -------- | :------- |
| userId | 用户Id |
| nickName | 昵称 |
| audioTrack | 音频轨道 |
| videoTrack | 视频轨道 |

  * 取消订阅。通过unSubscribeStreams方法可以取消订阅流。
  ```js
  JRTCRoom.unSubscribeStreams(streamIds);  // 取消订阅流，streamIds - 流ID Array
  ```

7.退出房间。
  ```js
  JWebrtc.exitRoom()
      .then(() => {
        // 退出房间成功
        JWebrtc.disconnectAll();  // 断开通信连接，销毁房间
        JWebrtc = null;           // 全局变量 JWebrtc 设置为null
      })
  ```

您可以下载示例代码，快速跑通Demo，实现频道内和其他人进行实时音视频通话，详情请参见[Web Demo](https://help.aliyun.com/document_detail/140519.html?spm=a2c4g.11186623.2.15.59c060cbnnmSoH#task-2312851)。
接口方法详情请参见[JRTC接口](#h5-api)。

# H5-API
import JRTCClient from '../lib/jrtc.min.js'

## 1.JRTCClient 

## init(JRTCInitParams)

const JWebrtc = JRTCClient.init(JRTCInitParams:{ appId: string })
参数：initParam <br/>
返回:  JRTCClient实例 <br/>
JRTCInitParams:  <br/>

| 参数     |  类型   |   说明 |
| -------- | :------- | -------: |
| appId | string | appId |

### setVideoEncodingParam()

JWebrtc.setVideoEncodingParam(resolution: string) <br/>
设置分辨率 <br/>
参数： resolution <br/>

| 参数     |  说明   |   是否必填 |
| -------- | :------- | -------: |
| resolution | resolution nhd(640*360)、hd(1280*720)、fhd(1920*1080) | Y |
### setAudioCodecOptions()

JWebrtc.setSudioCodecOptions(isStereo: Boolean) <br/>
参数： isStereo 是否开启立体声：true开启，false关闭<br/>
返回：无

### getVideoTrack()

JWebrtc.getVideoTrack()<br/>
参数： 无<br/>
说明：获取视频track<br/>
返回：videoTrack

### getAudioTrack()

JWebrtc.getAudioTrack()<br/>
参数： 无<br/>
说明：获取音频track<br/>
返回：audioTrack<br/>

### getScreenTrack()

JWebrtc.getScreenTrack()<br/>
说明： 获取屏幕共享track<br/>
参数： 无<br/>
返回：screenTrack
### enterRoom()

JWebrtc.enterRoom(enterRoomInfo)<br/>
说明：进入房间成功, 会返回一个ROOM对象: JRTCRoom,<br/>
参数：enterRoomInfo Object如下

| 参数     | 类型     |  是否必填   |   说明 |
| -------- | :------- | -------: |-------: |
| appId | string | Y | 应用ID，控制台获取 |
| token | string | Y | 用户生成token，生成方式参见XXXX取 |
| userId | string or number | Y | 用户Id |
| nonce | string | Y | 令牌随机码，用户生成 |
| timeStamp | string | Y | 令牌过期时间，用户生成 |
| roomId | number | Y | roomId |
| nickname | string | N | nickname |
| subscribeType | string | N | 大房间模式下，音频订阅模式： 1 固定订阅 2 普通订阅 。 默认为 1 |
| useVp8 | string | N | 是否开启vp8环境，默认false |
### exitRoom()

JWebrtc.exitRoom(roomId: string,userId: string)<br/>
说明：退出房间<br/>
参数： roomId、userId<br/>
返回：无
### 错误监听

JWebrtc.on('onError', (err) => {})<br/>
说明：错误事件，返回 { errorCode: XXX, message: XXXX}<br/>
错误信息参考：

| 类型     | errorCode     |  message   |   说明 |
| -------- | :------- | -------: |-------: |
| 鉴权错误 | 12000 | 'Authentication failure.' | 鉴权错误 |
| 网络错误 | 11000 |network_error | 网络已断开 |
| 音频错误 | 10001 | audio device not found | 未找到音频设备 |
|  |10003 | audio device not allowed |浏览器禁用音频设备|
| | 10005 | audio device not readable | 系统禁用音频设备 |
|视频错误| 10002| video device not found | 未找到视频设备|
|| 10004| video device not allowed |浏览器禁用视频设备|
|| 10006| video device not readable |系统禁用视频设备|
|屏幕共享| 10010 | "unknown screenshare error" |屏幕共享未知错误|
||10011| |screenshare not allowed| 屏幕共享被禁用|
||10012| screenshare ended |屏幕共享已取消|
|连接错误| 10030 | io server disconnect |服务断开|
||10031| io client disconnect| 客户端断开|
||10032| ping timeout |客户端连接超时|
||10033| transport close |连接关闭|
||10034| transport error |连接错误|
|其它错误| 10007| "constraints " + e.constraint + " error" OverConstrainedError|无法满足要求错误|
||1e4| e.message : "device unknown error"| 未知错误|
## 2.JRTCRoom
initBroadcast
调用JWebrtc.enterRoom 进入房间成功, 会返回一个ROOM对象: JRTCRoom,<br/>
1. 可通过JRTCRoom来进行房间内操作<br/>
2. 可直接通过调用JRTCRoom = await JWebrtc.enterRoom(XXXX)加入房间成功后的返回值来获取<br/>
也可在加入房间成功后，调用JRTCRoom = JWebrtc.getRoomObj() 来获取

### publishVideoStream()

const { track, streamId } = await JRTCRoom.publishVideoStream(videoTrack)<br/>
说明：发布视频流<br/>
参数： videoTrack<br/>
返回：track及streamId

### publishAudioStream()

const { track, streamId } = await JRTCRoom.publishAudioStream(audioTrack)<br/>
说明：发布音频<br/>
参数： audioTrack<br/>
返回：track及streamId

### unPublishStream()

JRTCRoom.unPublishStream(streamId)<br/>
说明：取消发布<br/>
参数： streamId<br/>
返回：无

### subscribeStreams()


JRTCRoom.subscribeStreams(streamIds: string[])<br/>
说明：订阅流，订阅成功如果产生新的消费者需手动监听streamSubscribed<br/>
参数： streamIds: string[]<br/>
返回：无<br/>
监听：streamSubscribed，返回peerInfo, peerInfo信息如下<br/>
JRTCRoom.on('StreamSubscribed', ({ streamInfo}) => {
 console.log(stream}) Info)

| 参数     |  说明   |
| -------- | :------- |
| userId | 用户Id |
| nickName | 昵称 |
| audioTrack | audioTrack |
| videoTrack | videoTrack |

### unSubscribeStreams()

JRTCRoom.unSubscribeStreams(streamIds: string[])<br/>
说明：取消订阅<br/>
参数： streamIds: string[]<br/>
返回：无

### pausePublish()

JRTCRoom.pausePublish(streamId)<br/>
说明：暂停发布<br/>
参数： streamId<br/>
返回：无

### resumePublish()

JRTCRoom.resumePublish(streamId)<br/>
说明：恢复发布<br/>
参数： streamId<br/>
返回：无

### pauseSubscribe()

JRTCRoom.pauseSubscribe(streamId)<br/>
说明：暂停订阅<br/>
参数： streamId<br/>
返回：无

### resumeSubscribe()

JRTCRoom.resumeSubscribe(streamId)<br/>
说明：恢复订阅<br/>
参数： streamId<br/>
返回：无


### sendMessage()

JRTCRoom.sendMessaage({msg: string, userId?: string})<br/>
说明：发送消息，可发送给指定人，可发送消息到房间<br/>
参数：msg: 消息体， userId: 指定userId<br/>
返回：无

### changeNickName()

JRTCRoom.changeNickName(nickName: string)<br/>
说明：修改昵称<br/>
参数： nickName<br/>
返回：无<br/>
监听：NickNameUpdate, <br/>
返回  {userRoomId: string, userId: string, nickName: string}

### enableStreamStat()

JRTCRoom.enableStreamStat(isStat, intervalSec)<br/>
说明： 监控接口<br/>
参数：

| 参数     |  说明   |   类型   |
| -------- | :------- |:------- |
| isStat | 是否开启监控 | boolean |
| interValSec | 几秒获取一次监控信息 | number |

返回：{ JRTCNetStats, JRTCLocalStreamStats, JRTCRemoteStreamStats }

| 参数     |  说明   |   类型   |
| -------- | :------- |:------- |
| JRTCNetStats | 基本信息 | ```{localIp：XX，serverIp： XX}``` |
| JRTCLocalStreamStats | 本地流信息 | ```{userId: XX,streamId: XX,kind: XX,rtt: XX,netLoss: XX,frameWidth: XX,frameHeight: XX,fps: XX}``` |
| JRTCRemoteStreamStats  | 远端流信息 | ``` { userId: XX,streamId: XX,kind: XX,jitterBufferDelay: XX,netLoss: XX,jitterBufferDelay: XX,netLoss: XX,frameWidth: XX,frameHeight: XX,fps: XX}```|

### getFixedAudioConsumers()

JRTCRoom.getFixedAudioConsumers()<br/>
说明：如果房间为大房间，可获取大房间固定音频， 可自行订阅<br/>
参数： 无<br/>
返回：fixedAudioConsumerList<br/>
fixedAudioConsumerList：<br/>

| 参数     |  说明   |
| -------- | :------- |
| streamId | 流ID |
| kind | 媒体类型 |
| track | 音频track |

### 会控相关接口

#### removePeer

JRTCRoom.removePeer(userId, appData)<br/>
说明：移除指定用户<br/>
参数说明：<br/>

| 参数     |  类型   |   是否必须   |  说明 |
| -------- | :------- |:------- |:------- |
|userId| string| Y |移除目标用户ID|
|appData| Object| N, 默认为 {} |自定义参数|

#### muteAudio

JRTCRoom.muteAudio({userId, appData})<br/>
说明：广播 房间内全局静音/指定用户静音<br/>
参数说明：<br/>

| 参数     |  类型   |   是否必须   |  说明 |
| -------- | :------- |:------- |:------- |
|userId| string| N| 目标用户ID，如果传入userId则为指定用户操作，不传则是对房间全局操作|
|appData| Object| N ,默认为 {} |自定义参数|

#### closeVideo

JRTCRoom.closeVideo()<br/>

##### 说明：广播 房间内全局关闭视频/关闭指定用户视频<br/>

参数说明：同muteAudio<br/>

#### forbidChat

JRTCRoom.forbidChat(appData)<br/>
说明：广播房间禁言<br/>
参数说明：<br/>

| 参数     |  类型   |   是否必须   |  说明 |
| -------- | :------- |:------- |:------- |
|appData| Object| N ,默认为 {} |自定义参数|
#### unForbidChat

JRTCRoom.unForbidChat()<br/>
说明：广播房间解除禁言<br/>
参数：同forbidChat<br/>

#### customSignal

JRTCRoom.customSignal()<br/>
说明：广播用户自定义信令<br/>
参数说明：

| 参数     |  类型   |   是否必须   |  说明 |
| -------- | :------- |:------- |:------- |
|eventName| string| Y| 自定义信令名称|
|userId| string| N |目标用户ID|
|appData| Object| N ,默认为 {} | 自定义参数|

### JRTCRoom房间内消息监听

#### UserJoinedRoom
```js
JRTCRoom.on('UserJoinRoom', (data: {userId:  string, nickName: string}) => {})
```
说明：加入房间

#### UserLeavedRoom
```js
JRTCRoom.on('UserLeaveRoom', (data: {userId:  string, nickName: string}) => {})
```
说明：离开房间

#### NickNameUpdated
```js
JRTCRoom.on('NickNameUpdate', (data: {userRoomId: string, userId:  string, nickName: string}) => {})
```
说明：用户修改昵称

#### MessageRecived
```js
JRTCRoom.on('MessageRecived', (data: {userId: string, nickName: string, msg: string}) => {})
```
说明：有成员发送消息时

| 参数     |  说明   |
| -------- | :------- |
|userId| 发送者id| 
|nickName| 发送者昵称|
|msg|消息体|

#### StreamPublished
```js
JRTCRoom.on('StreamPublished', (streamsInfo: streamInfo[]) => {
let streamIds = streamInfos.map(streamInfo => streamInfo.streamId);
      console.log('StreamPublished', streamIds)
      JRTCRoom.subscribeStreams(streamIds) // 可选择订阅
    })
```
说明：有新流发布<br/>
streamsInfo:,已发布流数组 streamInfo[]: streamInfo信息如下:

| 参数     |  说明   |
| -------- | :------- |
|userId| 用户Id| 
|streamId| 流ID|
|kind|流类型|
|streamName|流名称|
#### StreamUnpublished
```js
JRTCRoom.on('StreamUnpublished', (streamInfo: StreamInfo) => {
      console.log('StreamUnpulished', streamInfo) // 返回取消发布的流信息
    })
```
说明：当有成员取消音频或视频发布时<br/>
StreamInfo: 

| 参数     |  说明   |
| -------- | :------- |
|userId| 用户Id| 
|streamId| 流ID|
|kind|流类型|
#### StreamPaused
```js
JRTCRoom.on('StreamPaused', (streamInfo: StreamInfo) => {
      console.log('StreamPaused', streamInfo) // // 返回目标消费信息
    })
```
说明：当有成员暂停音频或视频发布时，返回StreamInfo信息如下<br/>

| 参数     |  说明   |
| -------- | :------- |
| userRoomId | 房间ID |
|userId| 暂停流发布的用户Id| 
|streamId| 流ID|
|kind|流类型|

#### StreamResumed
```js
JRTCRoom.on('StreamResumed', (streamInfo: StreamInfo) => {
      console.log('StreamResumed', streamInfo) // 返回目标消费信息
    })
```
说明：当有成员恢复音频或视频发布时，返回StreamInfo信息如下<br/>
##### 参数 说明
| 参数     |  说明   |
| -------- | :------- |
| userRoomId | 房间ID |
|userId| 恢复流发布的用户Id| 
|streamId| 流ID|
|kind|流类型|

#### 会控相关监听

#### UserRemoved
```js
JRTCRoom.on('UserRemoved', (data: {userId:  string, nickName: string}) => {})
```
说明：某个用户被移出房间

#### AudioMuted
```js
JRTCRoom.on('AudioMuted', (data: {userId:  string, nickName: string}) => {})

```
说明：某个用户被静音

#### RoomAudioMuted
```js
JRTCRoom.on('RoomAudioMuted', (data: {userId:  string, nickName: string}) => {})
```
说明：房间被静音

#### VideoClosed
```js
JRTCRoom.on('VideoClosed', (data: {userId:  string, nickName: string}) => {})
```
说明：某个用户视频被关闭

#### RoomVideoClosed
```js
JRTCRoom.on('RoomVideoClosed', (data: {userId:  string, nickName: string}) => {})
```
说明：房间被关闭所有视频

#### RoomChatForbidden
```js
JRTCRoom.on('RoomChatForbidden', (data: {userId:  string, nickName: string}) => {})
```
说明：房间禁止所有文字聊天

#### RoomChatUnForbidden
```js
JRTCRoom.on('RoomChatForbiddenlosed', (data: {userId:  string, nickName: string}) => {})
```
说明：房间取消禁言

#### SignalCustom
```js
JRTCRoom.on('RoomChatForbiddenlosed', (data: {userId:  string, nickName: string, eventName}) => {})
说明：用户自定义信令
```
