import React, { Component } from "react";
import { Form, Select, Input, Radio, Layout } from "antd";
import JRTC from "../common/JRTC";

const { Search } = Input;
const { Option } = Select;

const { Content } = Layout;
const layout = {
  labelCol: { span: 5 },
  wrapperCol: { span: 19 },
};
const ResolutionOptions = [
  { label: '640*360', val: 'nhd' },
  { label: '1280*720', val: 'hd' },
  { label: '1920*1080', val: 'fhd' },
  { label: '360*640', val: 'mnhd' },
  { label: '720*1280', val: 'mhd' },
  { label: '1080*1920', val: 'mfhd' }
]

const FpsOptions = [
  { label: 15, val: 15 },
  { label: 20, val: 20 },
  { label: 25, val: 25 },
]

class MeForm extends Component {
  constructor(props) {
    super(props);
    this.resolution = 'hd'
    this.fps = 25
  }
  onChange(e) {
    const { handleMeAudioStereo } = this.props
    handleMeAudioStereo(e.target.value)
  }

  changeNickName(val) {
    if (!val) {
      return;
    }
    JRTC.changeNickName(val)
    this.nickName.state.value = ''
  }

  sendMsg(val) {
    if (!val) {
      return;
    }
    JRTC.sendMsg(val)
    this.msgEl.state.value = ''
    this.msgEl.blur()
  }

  handleChange(resolution, fps) {
    const { handleResolution } = this.props
    this.resolution = resolution
    this.fps = fps
    handleResolution(resolution, fps)
  }
  render() {
    const { isDisabled, produceIceCandidates } = this.props
    return (
      <Content style={{ marginTop: 10 }}>
        <Search
          ref={el => this.msgEl = el}
          placeholder="广播消息……"
          enterButton="发送"
          allowClear={true}
          size="default"
          onSearch={value => this.sendMsg(value)}
        />

        <Search
          ref={el => this.nickName = el}
          style={{ marginTop: 10 }}
          placeholder="要修改的昵称"
          enterButton="修改"
          allowClear={true}
          size="default"
          onSearch={value => this.changeNickName(value)}
        />

        <h3 style={{ marginTop: 10 }}>ip: {produceIceCandidates.ip} &nbsp; port: {produceIceCandidates.port}</h3>

        <Form
          {...layout}
          name="basic"
          initialValues={{ isStereo: false, resolution: this.resolution, fps: this.fps }}
          style={{ marginTop: 10 }}
        >
        
          <Form.Item
            label="立体声"
            name="isStereo"
            className='formItem'
          >
            <Radio.Group onChange={(val) => this.onChange(val)} >
              <Radio value={false}>关闭</Radio>
              <Radio value={true}>开启</Radio>
            </Radio.Group>
          </Form.Item>
          
          <Form.Item
            label="分辨率"
            name="resolution"
            className='formItem'
          >
            <Select disabled={isDisabled} style={{ width: 120, margin: 5 }} onChange={(val) => this.handleChange(val, this.fps)}>
              {ResolutionOptions.map((item, index) =>
                <Option key={index} value={item.val}>{item.label}</Option>
              )}
            </Select>
          </Form.Item>
          <Form.Item
            label="帧率"
            name="fps"
            className='formItem'
          >
            <Select disabled={isDisabled} style={{ width: 120, margin: 5 }} onChange={(val) => this.handleChange(this.resolution, val)}>
              {FpsOptions.map((item, index) =>
                <Option key={index} value={item.val}>{item.label}</Option>
              )}
            </Select>
          </Form.Item>
        </Form>
      </Content>
    );
  }
}

export default MeForm;
