import React, { Component } from "react";
import { Row, Col, Button, Tooltip } from "antd";
import JRTC from "../common/JRTC";
import MeForm from '../component/meForm';
import {
  CloseOutlined,
  DesktopOutlined,
  AudioOutlined,
  AudioMutedOutlined,
  VideoCameraFilled,
  VideoCameraOutlined,
} from "@ant-design/icons";

class MeVideo extends Component {
  constructor(props) {
    super(props);
    this.state = {
      audioIconTitle: "静音",
      videoIconTitle: "关闭摄像头",
      screenIconTitle: "屏幕共享",
      audioIcon: <AudioOutlined />,
      videoIcon: <VideoCameraFilled />,
      screenIcon: <DesktopOutlined />,
      videoIconDisabled: false,
      audioStatus: true,
      videoStatus: true,
      screenStatus: false,
    };
    this.publishVideoStreamId = "";
    this.publishAudioStreamId = "";
  }
  componentDidMount() {
    const self = this
    JRTC.onMsg((data, type) => {
      self.reciveMsg(data, type)
    })
  }
  reciveMsg(data, type) {
    const self = this
    switch (type) {
      case 10012:
        self.operateScreenHandler()
        break
      default:
    }
  }

  async initMeVideo() {
    const self = this;
    if (self.state.videoStatus) {
      JRTC.publishVideoStream().then(res => {
        self.publishVideoStreamId = res.streamId;
        if (res.track) {
          self.setTracks(null, res.track).catch(reason => {
              console.error(reason);
          });
      }}).catch(err=> {console.log(err)});
    }
    if (self.state.audioStatus) {
      JRTC.publishAudioStream().then(res => {
        self.publishAudioStreamId = res.streamId;
      }).catch(err=> {console.log(err)});
    }
  }

  async setTracks(audioTrack, videoTrack){
    const { consumeVideo, consumeAudio } = this.refs;
    if (audioTrack) {
      const stream = new MediaStream();
      stream.addTrack(audioTrack);
      consumeAudio.srcObject = stream;
      await consumeAudio.load();
    } else {
      consumeAudio.srcObject = null;
    }
    if (videoTrack) {
      consumeVideo.srcObject = null;
      const stream = new MediaStream();
      stream.addTrack(videoTrack);
      consumeVideo.srcObject = stream;
      await consumeVideo.load();
      // consumeVideo.onloadedmetadata = () => consumeVideo.play()
    } else {
      consumeVideo.srcObject = null;
    }
  };

  // 大房间模式下，固定音频处理，handleFixedAudioTrack
  handleFixedAudioTrack(audioTracks) {
    if (!audioTracks || audioTracks.length < 1) {
      return;
    }
    for (let i = 0; i < 6; i++) {
      const consumeAudio = this.refs['consumeAudio' + i];
      if (audioTracks[i] && consumeAudio) {
        const stream = new MediaStream();
        stream.addTrack(audioTracks[i].track);
        consumeAudio.srcObject = stream;
        // consumeAudio.play();
        consumeAudio.load();
      }
    }
  }

  async operateScreenHandler() {
    const self = this;
    const { screenStatus, videoStatus } = self.state;
    let screenIconTitle, screenIcon, status, videoIconDisabled;
    if (screenStatus) {
      JRTC.setVideoEncodingParam('nhd');
      JRTC.unPublishStream(self.publishVideoStreamId);
      screenIconTitle = "屏幕共享";
      screenIcon = <DesktopOutlined />;
      status = false;
      videoIconDisabled = false;
      if (videoStatus) {
        const res = await JRTC.publishVideoStream();
        self.publishVideoStreamId = res.streamId;
        if (res.track) {
          self.setTracks(null, res.track).catch(reason => {
              console.error(reason);
          });
        }
      }
      self.setState({ screenIconTitle, screenIcon, screenStatus: status, videoIconDisabled });
    } else {
      JRTC.setVideoEncodingParam('fhd');
      JRTC.getScreenTrack().then(async track => {
        JRTC.unPublishStream(self.publishVideoStreamId);
        const res = await JRTC.operateScreen(track);
        self.publishVideoStreamId = res.streamId;
        if (res.track) {
          self.setTracks(null, res.track).catch(reason => {
              console.error(reason);
          });
        }
        screenIconTitle = "关闭屏幕共享";
        screenIcon = <CloseOutlined />;
        status = true;
        videoIconDisabled = true;
        this.setState({ screenIconTitle, screenIcon, screenStatus: status, videoIconDisabled });
      }).catch(err => {
        console.log(err)
      })
    }
  }

  async operateVideoHandler() {
    const self = this;
    const { videoStatus } = self.state;
    let videoIconTitle, videoIcon, status;
    if (videoStatus) {
      videoIconTitle = "打开摄像头";
      videoIcon = <VideoCameraOutlined />;
      status = false;
      JRTC.unPublishStream(self.publishVideoStreamId);
      this.setState({ videoIconTitle, videoIcon, videoStatus: status });
    } else {
      const res = await JRTC.publishVideoStream();
      self.publishVideoStreamId = res.streamId;
      videoIconTitle = "关闭摄像头";
      videoIcon = <VideoCameraFilled />;
      status = true;
      if (res.track) {
        self.setTracks(null, res.track).catch(reason => {
            console.error(reason);
        });;
      }
      this.setState({ videoIconTitle, videoIcon, videoStatus: status });
    }
  }

  async operateAudioHandler() {
    const self = this;
    const { audioStatus } = self.state;
    let audioIconTitle, audioIcon, status;
    if (audioStatus) {
      audioIconTitle = "取消静音";
      audioIcon = <AudioMutedOutlined />;
      status = false;
      JRTC.unPublishStream(self.publishAudioStreamId);
      this.setState({ audioIconTitle, audioIcon, audioStatus: status });
    } else {
      const res = await JRTC.publishAudioStream();
      self.publishAudioStreamId = res.streamId;
      audioIconTitle = "静音";
      audioIcon = <AudioOutlined />;
      status = true;
      this.setState({ audioIconTitle, audioIcon, audioStatus: status });
    }
  }

  pausePublish(streamId) {
    JRTC.pausePublish(streamId)
  }
  resumePublish(producerId) {
    JRTC.resumePublish(producerId)
  }

  /** 修改分辨率、帧率 */
  async handleResolution(resolution, fps) {
    const self = this;
    JRTC.setVideoEncodingParam(resolution, fps)
    await JRTC.unPublishStream(self.publishVideoStreamId);
    const res = await JRTC.publishVideoStream();
    self.publishVideoStreamId = res.streamId;
    if (res.track) {
      self.setTracks(null, res.track).catch(reason => {
          console.error(reason);
      });
    }
  }
  /* 音视频操作 */
  async handleMeAudioStereo(val) {
    const self = this;
    JRTC.setAudioStereo(val);
    JRTC.unPublishStream(self.publishAudioStreamId);
    const res = await JRTC.publishAudioStream();
    self.publishAudioStreamId = res.streamId;
  }

  getFixChannelAudios() {
    const me = this;
    const { roomType } = me.props;
    let result = [];
    if (roomType !== 2) {
      return result;
    }
    for (let i = 0; i < 6; i++) {
      result.push(<audio
        ref={'consumeAudio' + i}
        key={i}
        autoPlay
        controls={true}
      />);
    }
    return (<div>
      {result}
    </div>);
  }

  render() {
    const self = this;
    const { nickName, produceIceCandidates, peerId, roomId } = this.props;
    const {
      screenStatus,
      audioIconTitle,
      audioIcon,
      videoIconTitle,
      videoIcon,
      screenIconTitle,
      screenIcon,
      videoIconDisabled,
    } = self.state;
    const fixChannelAudios = self.getFixChannelAudios();
    return (
      <div style={{ width: '100%' }}>
        <div className="icons-list">
          <Tooltip title="昵称">
            <b>{nickName || "本人"}</b>
          </Tooltip>
          <Tooltip title={screenIconTitle}>
            <Button
              shape="circle"
              size="small"
              style={{ margin: 5 }}
              icon={screenIcon}
              onClick={self.operateScreenHandler.bind(self)}
            />
          </Tooltip>
          <Tooltip title={audioIconTitle}>
            <Button
              shape="circle"
              size="small"
              style={{ margin: 5 }}
              icon={audioIcon}
              onClick={self.operateAudioHandler.bind(self)}
            />
          </Tooltip>
          <Tooltip title={videoIconTitle}>
            <Button
              disabled={videoIconDisabled}
              shape="circle"
              size="small"
              icon={videoIcon}
              onClick={self.operateVideoHandler.bind(self)}
            />
          </Tooltip>
          <Button
              type="text"
              size="small"
            >peerId: {peerId}</Button>
            <Button
              type="text"
              size="small"
            >roomId: {roomId}</Button>
        </div>
        {/* width={400}
        height={250} */}
        <Row type="flex" justify="start">
          <Col span={24}>
            <video
              width="100%"
              height={300}
              autoPlay
              muted
              playsInline
              controls={false}
              ref="consumeVideo"
            >
              不支持
            </video>
            <audio ref="consumeAudio" muted playsInline autoPlay controls={false} />
          </Col>
          <Col span={24}>
            <div className="icons-list" style={{ marginBottom: 10, position: 'absolute', bottom: 30 }}>
              <Tooltip title='暂停音频发布'>
                <Button ghost shape="circle" icon={<AudioMutedOutlined />} size="small" style={{ margin: 5 }} onClick={() => self.pausePublish(self.publishAudioStreamId)}></Button>
              </Tooltip>
              <Tooltip title='恢复音频发布'>
                <Button ghost shape="circle" icon={<AudioOutlined />} size="small" style={{ margin: 5 }} onClick={() =>  self.resumePublish(self.publishAudioStreamId)}></Button>
              </Tooltip>
              <Tooltip title='暂停视频发布'>
              <Button ghost shape="circle" icon={<VideoCameraOutlined />} size="small" style={{ margin: 5 }} onClick={() => self.pausePublish(self.publishVideoStreamId)}></Button>
              </Tooltip>
              <Tooltip title='恢复视频发布'>
                <Button ghost shape="circle" icon={<VideoCameraFilled />} size="small" style={{ margin: 5 }} onClick={() => self.resumePublish(self.publishVideoStreamId)}></Button>
              </Tooltip>
            </div>
          </Col>
        </Row>
        
        <MeForm ref='meForm' handleMeAudioStereo={self.handleMeAudioStereo.bind(self)} handleResolution={self.handleResolution.bind(self)} isDisabled={screenStatus} produceIceCandidates={produceIceCandidates} />
        {fixChannelAudios}
      </div>
    );
  }
}

export default MeVideo;
