import React, { PureComponent } from "react";
import { Row, Col, Button, Tooltip, Input,  } from 'antd';
import JRTC from '../common/JRTC'
import {
  AudioOutlined,
  AudioMutedOutlined,
  VideoCameraFilled,
  VideoCameraOutlined,
  DeleteOutlined
} from '@ant-design/icons';

const { Search } = Input;

class PeerVideo extends PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      audioIconTitle: "取消音频订阅",
      videoIconTitle: "取消视频订阅",
      audioIcon: <AudioOutlined />,
      videoIcon: <VideoCameraFilled />,
      audioStatus: true,
      videoStatus: true,
      nickName: ''
    }
    this.audioStreamId = null
    this.videoStreamId = null
  }

  setNickName(nickName) {
    const self = this
    self.setState({nickName: nickName})
  }
  componentDidMount() {
    const { audioTrack, videoTrack, audioStreamId, videoStreamId, nickName } = this.props;
    if(nickName) {
      this.setNickName(nickName)
    }
    this.setTracks(audioTrack, videoTrack, audioStreamId, videoStreamId,);
  }

  componentDidUpdate() {
    const { audioTrack, videoTrack, audioStreamId, videoStreamId } = this.props;
    this.setTracks(audioTrack, videoTrack, audioStreamId, videoStreamId).catch(reason => {
        console.error(reason);
    });
  }

  async setTracks(audioTrack, videoTrack, audioStreamId, videoStreamId) {
    const { consumeVideo, consumeAudio } = this.refs;
    if (audioStreamId) {
      this.audioStreamId = audioStreamId;
    }
    if (videoStreamId) {
      this.videoStreamId = videoStreamId;
    }
    if (audioTrack) {
      const stream = new MediaStream();
      stream.addTrack(audioTrack);
      consumeAudio.srcObject = stream;
      await consumeAudio.load();
    } else {
      consumeAudio.srcObject = null;
      await consumeAudio.pause();
    }
    if (videoTrack) {
      const stream = new MediaStream();
      stream.addTrack(videoTrack);
      consumeVideo.srcObject = stream;
      await consumeVideo.load();
    } else {
      consumeVideo.srcObject = null;
      await consumeVideo.pause();
    }
  }

  operatePeerStreams(streamId, status) {
    if (status) {
      JRTC.subscribeStreams([streamId])
    } else {
      JRTC.unSubscribeStreams([streamId])
    }
  }

  operateAudioHandler() {
    const self = this;
    const { audioStatus } = self.state;
    let audioIconTitle, audioIcon, status;
    if (audioStatus) {
      audioIconTitle = '订阅音频';
      audioIcon = <AudioMutedOutlined />;
      status = false;
    } else {
      audioIconTitle = '取消音频订阅';
      audioIcon = <AudioOutlined />;
      status = true;
    }
    if (self.audioStreamId) {
      self.operatePeerStreams(self.audioStreamId, status);
    }
    this.setState({ audioIconTitle, audioIcon, audioStatus: status });
  }

  operateVideoHandler() {
    const self = this;
    const { videoStatus } = self.state;
    let videoIconTitle, videoIcon, status;
    if (videoStatus) {
      videoIconTitle = '订阅视频';
      videoIcon = <VideoCameraOutlined/>;
      status = false;
    } else {
      videoIconTitle = '取消视频订阅';
      videoIcon = <VideoCameraFilled/>;
      status = true;
    }
    if (self.videoStreamId) {
      self.operatePeerStreams(self.videoStreamId, status);
    }
    this.setState({videoIconTitle, videoIcon, videoStatus: status});
  }

  pauseSubscribe(streamId) {
    JRTC.pauseSubscribe(streamId)
  }
  resumeSubscribe(streamId) {
    JRTC.resumeSubscribe(streamId)
  }

  sendMsg(msg) {
    const {userId} = this.props
    JRTC.sendMsg(msg, userId)
    this.appointSendMsg.state.value = ''
    this.appointSendMsg.blur()
  }
  /* 会控 */
  removePeer() {
    const { userId } = this.props;
    JRTC.customSignal({userId: userId})
  }

  render() {
    const self = this;
    const { audioIconTitle, audioIcon, videoIconTitle, videoIcon, nickName } = self.state;
    
    return (
      <div style={{ width: '100%' }}>
        <div className="icons-list">
          <Tooltip title="昵称">
            <b ref="nickName">{nickName || '游客'}</b>
          </Tooltip>
          <Tooltip title={audioIconTitle}>
            <Button shape="circle" size="small" style={{ margin: 5 }} icon={audioIcon}
              onClick={self.operateAudioHandler.bind(self)}
            />
          </Tooltip>
          <Tooltip title={videoIconTitle}>
            <Button shape="circle" size="small" icon={videoIcon} onClick={self.operateVideoHandler.bind(self)} />
          </Tooltip>
          <Tooltip title='会控'>
            <Button shape="circle" size="small" style={{ margin: 5 }} icon={<DeleteOutlined />} onClick={self.removePeer.bind(self)} />
          </Tooltip>
        </div>
        {/* width={400}
        height={250} */}
        <Row type="flex" justify="start">
          <Col span={24}>
            <video
              width="100%"
              height={300}
              autoPlay
              muted
              playsInline
              ref="consumeVideo">
              不支持
            </video>
            <audio
              ref='consumeAudio'
              playsInline
              autoPlay
              controls={false}
            />
          </Col>
        </Row>
        <div className="icons-list" style={{ marginBottom: 10, position: 'absolute', bottom: 30 }}>
          <Tooltip title='暂停音频订阅'>
            <Button ghost shape="circle" icon={<AudioMutedOutlined />} size="small" style={{ margin: 5 }} onClick={() => self.pauseSubscribe(self.audioStreamId)}></Button>
          </Tooltip>
          <Tooltip title='恢复音频订阅'>
            <Button ghost shape="circle" icon={<AudioOutlined />} size="small" style={{ margin: 5 }} onClick={() => self.resumeSubscribe(self.audioStreamId)}></Button>
          </Tooltip>
          <Tooltip title='暂停视频频订阅'>
           <Button ghost shape="circle" icon={<VideoCameraOutlined />} size="small" style={{ margin: 5 }} onClick={() => self.pauseSubscribe(self.videoStreamId)}></Button>
          </Tooltip>
          <Tooltip title='恢复视频订阅'>
            <Button ghost shape="circle" icon={<VideoCameraFilled />} size="small" style={{ margin: 5 }} onClick={() => self.resumeSubscribe(self.videoStreamId)}></Button>
          </Tooltip>
        </div>
        {/* ref='sendMsg' */}
        <Search
          ref={el => this.appointSendMsg = el}
          style={{width: '100%'}}
          placeholder="发送消息给当前用户……"
          enterButton="发送"
          size="small"
          allowClear={true}
          onSearch={value => self.sendMsg(value)}
      />
      </div>
    );
  }
}

export default PeerVideo;