import React, { Component } from "react";
// import md5 from "js-md5";
import { Col, notification, Row, Switch, Carousel, Button, Drawer  } from "antd";
import JRTC from "../common/JRTC";
import TokenUtil from "../common/TokenUtil";
import Util from "../common/util";
import { config } from "../common/config";
import MeVideo from "./meVideos.js";
import PeerVideo from "./peerVideo.js";
import Stats from "../component/stats.js";

import RoomControl from '../component/roomControl.js';

class Room extends Component {
	constructor(props) {
		super(props);
		this.state = {
			visible: false,
			nickName: '',
			isMsg: false,
			produceIceCandidates: {},
			stats: null,
			roomType: '',
			allPeers: [], // 所有参会人员
			page: 0,
			pageSize: 4,
			roomId: '', // demo使用
			peerId: '' // demo使用
		};
		this.userId = null
		this.JRTCRoom = null
		this.allPeersVideoStreams = new Map() // 所有参会人员视频信息(不展示在页面中的不订阅)
		this.renderPeers = [] // 当前页面显示的成员信息
	}
	streamSubscribed(peerInf) {
		const { allPeers } = this.state
		const ind = allPeers.findIndex(item => item.userId === peerInf.userId)
		if (ind > -1) {
			allPeers[ind] = { ...allPeers[ind], ...peerInf }
		} else {
			allPeers.push(peerInf)
		}
		this.setState({ allPeers })
	}

	handleUserJoinedRoom(data) {
		const self = this
		self.streamSubscribed(data)
	}

	handleUserLeave(data) {
		const self = this
		const { allPeers } = self.state
		this.setState({ allPeers: allPeers.filter(item => item.userId !== data.userId) })
	}

	handleNickNameUpdate(data) {
		const self = this
		if (self.userId === data.userId) {
			sessionStorage.setItem('nickName', data.nickName)
			this.setState({ nickName: data.nickName })
		} else {
			const { allPeers } = this.state
			const ind = allPeers.findIndex(item => item.userId === data.userId)
			if (ind > -1 && self.refs['peerVideo' + data.userId]) {
				self.refs['peerVideo' + data.userId].setNickName(data.nickName)
			}
		}
	}

	handleReciveStats(stats) {
		this.setState({ stats: stats })
	}

	handleStreamPublished(streamInfos) {
		if (streamInfos.length === 0) return
		const self = this
		const { allPeers, page, pageSize } = this.state
		const audioAndrenderVideoStreams = []
		self.renderPeers = allPeers.slice((page - 1) * pageSize, page * pageSize)
		streamInfos.map(streamInfo => {
			// 收到流发布消息，如果为音频则全部订阅，如果为视频且该成员在显示范围内，也订阅
			if (streamInfo.kind === 'audio' || self.renderPeers.find(item => item.userId === streamInfo.userId)) {
				audioAndrenderVideoStreams.push(streamInfo.streamId)
			}
			// 如果为视频则不订阅直接保存，待切换分页时订阅显示成员视频
			if (streamInfo.kind === 'video') {
				self.allPeersVideoStreams.set(streamInfo.userId, {
					userId: streamInfo.userId,
					peeerId: streamInfo.peerId,
					streams: streamInfo.streamId
				})
			}
			return []
		})
		self.JRTCRoom.subscribeStreams(audioAndrenderVideoStreams);
	}

	handleStreamUnPublished(streamInfo) {
		const self = this
		if (streamInfo.kind === 'video') {
			if (self.allPeersVideoStreams.has(streamInfo.userId)) {
				self.allPeersVideoStreams.delete(streamInfo.userId)
			}
		}
	}
	
	handleUserRemoved(data) {
		notification.info({
			message: '提示',
			description: '您已被【' + data.nickName + '】移出房间',
			placement: 'topRight'
		});
	}
	reciveMsg(data, type) {
		const self = this
		console.log(type, JSON.stringify(data))
		switch (type) {
			case 'onError':
				notification.error({
					message: 'errorCode: ' + data.errorCode,
					description: data.message,
					placement: 'topRight'
				});
				break
			case 'UserJoinedRoom':
				notification.info({
					message: '提示',
					description: '用户【' + data.nickName + '】加入房间',
					placement: 'topRight'
				});
				self.handleUserJoinedRoom(data)
				break
			case 'UserLeavedRoom':
				notification.info({
					message: '提示',
					description: '用户【' + data.nickName + '】离开房间',
					placement: 'topRight'
				});
				self.handleUserLeave(data)
				break
			case 'StreamPublished':
				self.handleStreamPublished(data)
				break
			case 'StreamUnpublished':
				self.handleStreamUnPublished(data)
				break
			case 'StreamSubscribed':
				self.streamSubscribed(data)
				break
			case 'NickNameUpdated':
				self.handleNickNameUpdate(data)
				break
			case 'MessageRecived':
				notification.info({
					message: '来自【' + data['nickName'] + '】的消息',
					description: data.msg
				});
				break
			case 'onStats':
				self.handleReciveStats(data)
				break
			case 'UserRemoved':
				self.handleUserRemoved(data)
				break
			case 'AudioMuted':
				notification.info({
					message: '您已被【' + data.nickName + '】静音',
					description: data.data,
					placement: 'topRight'
				});
				break
			case 'RoomAudioMuted':
				notification.info({
					message: '该房间已被【' + data.nickName + '】设置为全局静音',
					description: data.data,
					placement: 'topRight'
				});
				break
			case 'VideoClosed':
				notification.info({
					message: '您已被【' + data.nickName + '】关闭视频',
					description: data.data,
					placement: 'topRight'
				});
				break
			case 'RoomVideoClosed':
				notification.info({
					message: '该房间已被【' + data.nickName + '】全局关闭视频',
					description: data.data,
					placement: 'topRight'
				});
				break
			default:
				// console.log(type, data)
		}
	}

	async initRoom() {
		const self = this;
		const { roomId, userId, nickName, isMsg = false, subscribeType = 1, isVideoRecvonly = false, useVp8 = false } = Util.decodeQuery(
			this.props.location.search
		);
		if (!roomId || !userId) {
			this.props.history.push({ pathname: "/" });
			return;
		}
		this.userId  = userId;
		this.nickName = sessionStorage.getItem('nickName') || nickName;

		const appId = config.appId

		// const userId = md5(nickName)
		const nonce = "AK-" + userId;
		const timestamp = 4762379647000;
		const token = TokenUtil.hmacSha256(appId, config.appKey, userId, roomId, nonce, timestamp);

		await JRTC.init(appId)

		this.setState({ nickName: self.nickName, isMsg })
		// setVideoRecvonly方法 如有需要，则需在加入房间前设置
		JRTC.setVideoRecvonly(JSON.parse(isVideoRecvonly))

		const roomObj = await JRTC.enterRoom({
			appId,
			token,
			userId,
			nonce,
			timestamp,
			userRoomId: roomId,
			nickName: self.nickName,
			subscribeType,
			useVp8: JSON.parse(useVp8) // 是否使用vp8,布尔值，默认false
		}).catch(err => {
			if (err.msg) {
				notification.error({
					message: '提示',
					description: err.msg || err.msg.message || 'enterRoom error',
					placement: 'topRight'
				});
			}
			return
		});
		if (!roomObj) return
		self.JRTCRoom = roomObj
		// roomObj.peers: 加入房间时，已进入房间人员信息
		self.setState({
			allPeers: roomObj.peers.filter(item => item.userId !== userId),
			produceIceCandidates: roomObj.produceIceCandidates[0] || {}
		})
		if (self.refs.meVideo) {
			self.refs.meVideo.initMeVideo();
		}

		// 如果为大房间，获取consumerList
		if (roomObj._roomTemplate.templateId === 2) {
			self.setState({ roomType: roomObj._roomTemplate.templateId })
			JRTC.getFixedAudioConsumers().then(res => {
				self.refs.meVideo.handleFixedAudioTrack(res)
			})
		}

		// 加入房间后，处理已发布流 roomObj.streamInfos
		self.handleStreamPublished(roomObj.streamInfos)

		// 加入房间后获取真实的peerId roomId----demo使用
		self.setState({ roomId: roomObj.roomId, peerId: roomObj.peerId})
	}

	componentDidMount() {

		const {isSupport, message, result} = JRTC.checkSystemRequirements()
    console.log(isSupport, message, result)
    if(!isSupport) {
      notification.error({
        message: '提示',
        description: '不支持当前浏览器',
        placement: 'topRight'
      });
    }
		

		const self = this;
		JRTC.onMsg((data, type) => {
			self.reciveMsg(data, type)
		})

		self.initRoom()
	}

	componentWillUnmount() {
		sessionStorage.removeItem('nickName')
		if (!JRTC.isInit) return
		JRTC.destroy();
	}

	// 统计
	getStats(val) {
		JRTC.getStats(val, 3)
		if (!val) {
			this.setState({ stats: null })
		}
	}

	// 切换面板后回调
	afterChange(cur) {
		const self = this
		const { allPeers, pageSize, page } = self.state
		if (cur === 0 || page === cur) return
		// 切换面板时，需要暂停该页已订阅的视频, 然后订阅或恢复订阅下一页视频
		self.renderPeers.map(item => {
			if (self.allPeersVideoStreams.has(item.userId)) {
				const peerInfo = self.allPeersVideoStreams.get(item.userId)
				JRTC.pauseSubscribe(peerInfo.streams)
			}
			return []
		})
		self.renderPeers = allPeers.slice((cur - 1) * pageSize, cur * pageSize)
		self.setState({ page: cur })
		self.renderPeers.map(item => {
			if (self.allPeersVideoStreams.has(item.userId)) {
				const peerInfo = self.allPeersVideoStreams.get(item.userId)
				if (item.videoTrack) {
					// 如果已经订阅过，则调用恢复订阅
					JRTC.resumeSubscribe(item.videoTrack.streamId)
				} else {
					// 如果从未订阅过，则调用订阅
					self.JRTCRoom.subscribeStreams([peerInfo.streams])
				}
			}
			return []
		})
	}

	// 退出房间
	exitRoom() {
		JRTC.destroy();
		this.props.history.push({
      pathname: "/"
    });
	}

	// 全部成员弹框
	showDrawer() {
		const self = this
		self.setState({visible: true})
	};

	onClose() {
		const self = this
		self.setState({visible: false})
	};


	render() {
		const self = this;
		const { pageSize, allPeers, nickName, produceIceCandidates, stats, roomType, roomId, peerId, visible } = self.state
		let result = [];
		for (let i = 0, len = allPeers.length; i < len; i += pageSize) {
			result.push(allPeers.slice(i, i + pageSize));
		}
		return (
			<>
				<Carousel ref="myCarousel"  arrows infinite={false} afterChange={(cur) => self.afterChange(cur)} className={ `${!Util.IsPC() ? 'isMob' : 'isPc'}` } style={{ background: '#ffffff', maxWidth: '860px', padding: '8px', margin: '0 auto' }} dots={false}>
					<Row gutter={8} justify="start">
						<Col span={24}>
							<MeVideo ref="meVideo" nickName={nickName} roomId={roomId} peerId={peerId} produceIceCandidates={produceIceCandidates}
								roomType={roomType} />
							<div style={{ padding: '0 10px 10px' }}>
								<span>统计：</span>
								<Switch checkedChildren="开启" unCheckedChildren="关闭" onChange={(val) => self.getStats(val)} />
								{stats ?
									(<Stats stats={stats} />) : null}
							</div>
						</Col>
					</Row>
					{result.map((it, index) => {
						return (
							<Row gutter={4} key={index} justify="start">
								{it.map((item, ind) => {
									return (
										<Col span={12} key={ind}>
											<PeerVideo ref={"peerVideo" + item.userId} key={item.userId}
												userId={item.userId}
												nickName={item.nickName}
												audioStreamId={item.audioTrack ? item.audioTrack.streamId : null}
												videoStreamId={item.videoTrack ? item.videoTrack.streamId : null}
												audioTrack={item.audioTrack ? item.audioTrack.track : null}
												videoTrack={item.videoTrack ? item.videoTrack.track : null} />
										</Col>)
								})}
							</Row>)
					})}
				</Carousel>
				<div className="roomControlBox">
					<Button  type="primary" onClick={() => self.showDrawer()}>全部成员</Button>
					<Button type="primary" onClick={() => self.exitRoom()}>退出房间</Button>
				</div>

				<Drawer
					title="全部成员"
					placement="bottom"
					closable={false}
					onClose={() => self.onClose()}
					visible={visible}
					height="80%"
			>
					<RoomControl allPeers={allPeers} />
			</Drawer>

			</>
		);
	}
}

export default Room;
